Package: shinychat
Title: Chat UI Component for 'shiny'
Version: 0.4.0
Authors@R: c(
    person("Joe", "Cheng", , "joe@posit.co", role = "aut"),
    person("Carson", "Sievert", , "carson@posit.co", role = "aut"),
    person("Garrick", "Aden-Buie", , "garrick@adenbuie.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7111-0077")),
    person("Barret", "Schloerke", , "barret@posit.co", role = "aut",
           comment = c(ORCID = "0000-0001-9986-114X")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Provides a scrolling chat interface with multiline input,
    suitable for creating chatbot apps based on Large Language Models
    (LLMs). Designed to work particularly well with the 'ellmer' R package
    for calling LLMs.
License: MIT + file LICENSE
URL: https://posit-dev.github.io/shinychat/r/,
        https://github.com/posit-dev/shinychat
BugReports: https://github.com/posit-dev/shinychat/issues
Imports: base64enc, bslib, cli, coro, ellmer (>= 0.4.1), fastmap,
        htmltools, jsonlite, lifecycle, promises (>= 1.3.2), rlang (>=
        1.1.0), S7, shiny (>= 1.10.0)
Suggests: covr, knitr, later, R6, testthat (>= 3.0.0), withr
Config/Needs/website: tidyverse/tidytemplate, rmarkdown, weathR, gt,
        glue, bsicons
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-06-01 16:51:50 UTC; cpsievert
Author: Joe Cheng [aut],
  Carson Sievert [aut],
  Garrick Aden-Buie [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7111-0077>),
  Barret Schloerke [aut] (ORCID: <https://orcid.org/0000-0001-9986-114X>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Garrick Aden-Buie <garrick@adenbuie.com>
Repository: CRAN
Date/Publication: 2026-06-01 19:10:02 UTC
Built: R 4.4.3; ; 2026-06-01 20:08:20 UTC; unix
