% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinychat-package.R
\docType{package}
\name{shinychat-package}
\alias{shinychat}
\alias{shinychat-package}
\title{shinychat: Chat UI Component for 'shiny'}
\description{
\if{html}{\figure{logo.svg}{options: style='float: right' alt='logo' width='120'}}

Provides a scrolling chat interface with multiline input, suitable for creating chatbot apps based on Large Language Models (LLMs). Designed to work particularly well with the 'ellmer' R package for calling LLMs.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://posit-dev.github.io/shinychat/r/}
  \item \url{https://github.com/posit-dev/shinychat}
  \item Report bugs at \url{https://github.com/posit-dev/shinychat/issues}
}

}
\author{
\strong{Maintainer}: Garrick Aden-Buie \email{garrick@adenbuie.com} (\href{https://orcid.org/0000-0002-7111-0077}{ORCID})

Authors:
\itemize{
  \item Garrick Aden-Buie \email{garrick@adenbuie.com} (\href{https://orcid.org/0000-0002-7111-0077}{ORCID})
  \item Joe Cheng \email{joe@posit.co}
  \item Carson Sievert \email{carson@posit.co}
  \item Barret Schloerke \email{barret@posit.co} (\href{https://orcid.org/0000-0001-9986-114X}{ORCID})
}

Other contributors:
\itemize{
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}
